/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __PLAYERCTL_GENERATED_H__
#define __PLAYERCTL_GENERATED_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.mpris.MediaPlayer2 */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2 (org_mpris_media_player2_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2, OrgMprisMediaPlayer2))
#define IS_ORG_MPRIS_MEDIA_PLAYER2(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2))
#define ORG_MPRIS_MEDIA_PLAYER2_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2, OrgMprisMediaPlayer2Iface))

struct _OrgMprisMediaPlayer2;
typedef struct _OrgMprisMediaPlayer2 OrgMprisMediaPlayer2;
typedef struct _OrgMprisMediaPlayer2Iface OrgMprisMediaPlayer2Iface;

struct _OrgMprisMediaPlayer2Iface
{
  GTypeInterface parent_iface;


  gboolean (*handle_quit) (
    OrgMprisMediaPlayer2 *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_raise) (
    OrgMprisMediaPlayer2 *object,
    GDBusMethodInvocation *invocation);

  gboolean  (*get_can_quit) (OrgMprisMediaPlayer2 *object);

  gboolean  (*get_can_raise) (OrgMprisMediaPlayer2 *object);

  const gchar * (*get_desktop_entry) (OrgMprisMediaPlayer2 *object);

  gboolean  (*get_has_track_list) (OrgMprisMediaPlayer2 *object);

  const gchar * (*get_identity) (OrgMprisMediaPlayer2 *object);

  const gchar *const * (*get_supported_mime_types) (OrgMprisMediaPlayer2 *object);

  const gchar *const * (*get_supported_uri_schemes) (OrgMprisMediaPlayer2 *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2, g_object_unref)
#endif

GType org_mpris_media_player2_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_mpris_media_player2_interface_info (void);
guint org_mpris_media_player2_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_mpris_media_player2_complete_raise (
    OrgMprisMediaPlayer2 *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_complete_quit (
    OrgMprisMediaPlayer2 *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void org_mpris_media_player2_call_raise (
    OrgMprisMediaPlayer2 *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_call_raise_finish (
    OrgMprisMediaPlayer2 *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_call_raise_sync (
    OrgMprisMediaPlayer2 *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_call_quit (
    OrgMprisMediaPlayer2 *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_call_quit_finish (
    OrgMprisMediaPlayer2 *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_call_quit_sync (
    OrgMprisMediaPlayer2 *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gboolean org_mpris_media_player2_get_can_quit (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_can_quit (OrgMprisMediaPlayer2 *object, gboolean value);

gboolean org_mpris_media_player2_get_can_raise (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_can_raise (OrgMprisMediaPlayer2 *object, gboolean value);

gboolean org_mpris_media_player2_get_has_track_list (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_has_track_list (OrgMprisMediaPlayer2 *object, gboolean value);

const gchar *org_mpris_media_player2_get_identity (OrgMprisMediaPlayer2 *object);
gchar *org_mpris_media_player2_dup_identity (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_identity (OrgMprisMediaPlayer2 *object, const gchar *value);

const gchar *org_mpris_media_player2_get_desktop_entry (OrgMprisMediaPlayer2 *object);
gchar *org_mpris_media_player2_dup_desktop_entry (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_desktop_entry (OrgMprisMediaPlayer2 *object, const gchar *value);

const gchar *const *org_mpris_media_player2_get_supported_uri_schemes (OrgMprisMediaPlayer2 *object);
gchar **org_mpris_media_player2_dup_supported_uri_schemes (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_supported_uri_schemes (OrgMprisMediaPlayer2 *object, const gchar *const *value);

const gchar *const *org_mpris_media_player2_get_supported_mime_types (OrgMprisMediaPlayer2 *object);
gchar **org_mpris_media_player2_dup_supported_mime_types (OrgMprisMediaPlayer2 *object);
void org_mpris_media_player2_set_supported_mime_types (OrgMprisMediaPlayer2 *object, const gchar *const *value);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PROXY (org_mpris_media_player2_proxy_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PROXY, OrgMprisMediaPlayer2Proxy))
#define ORG_MPRIS_MEDIA_PLAYER2_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PROXY, OrgMprisMediaPlayer2ProxyClass))
#define ORG_MPRIS_MEDIA_PLAYER2_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PROXY, OrgMprisMediaPlayer2ProxyClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PROXY))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PROXY))

typedef struct _OrgMprisMediaPlayer2Proxy OrgMprisMediaPlayer2Proxy;
typedef struct _OrgMprisMediaPlayer2ProxyClass OrgMprisMediaPlayer2ProxyClass;
typedef struct _OrgMprisMediaPlayer2ProxyPrivate OrgMprisMediaPlayer2ProxyPrivate;

struct _OrgMprisMediaPlayer2Proxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgMprisMediaPlayer2ProxyPrivate *priv;
};

struct _OrgMprisMediaPlayer2ProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_mpris_media_player2_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2Proxy, g_object_unref)
#endif

void org_mpris_media_player2_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2 *org_mpris_media_player2_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2 *org_mpris_media_player2_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_mpris_media_player2_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2 *org_mpris_media_player2_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2 *org_mpris_media_player2_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_SKELETON (org_mpris_media_player2_skeleton_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_SKELETON, OrgMprisMediaPlayer2Skeleton))
#define ORG_MPRIS_MEDIA_PLAYER2_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_SKELETON, OrgMprisMediaPlayer2SkeletonClass))
#define ORG_MPRIS_MEDIA_PLAYER2_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_SKELETON, OrgMprisMediaPlayer2SkeletonClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_SKELETON))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_SKELETON))

typedef struct _OrgMprisMediaPlayer2Skeleton OrgMprisMediaPlayer2Skeleton;
typedef struct _OrgMprisMediaPlayer2SkeletonClass OrgMprisMediaPlayer2SkeletonClass;
typedef struct _OrgMprisMediaPlayer2SkeletonPrivate OrgMprisMediaPlayer2SkeletonPrivate;

struct _OrgMprisMediaPlayer2Skeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgMprisMediaPlayer2SkeletonPrivate *priv;
};

struct _OrgMprisMediaPlayer2SkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_mpris_media_player2_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2Skeleton, g_object_unref)
#endif

OrgMprisMediaPlayer2 *org_mpris_media_player2_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.mpris.MediaPlayer2.Player */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER (org_mpris_media_player2_player_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER, OrgMprisMediaPlayer2Player))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER, OrgMprisMediaPlayer2PlayerIface))

struct _OrgMprisMediaPlayer2Player;
typedef struct _OrgMprisMediaPlayer2Player OrgMprisMediaPlayer2Player;
typedef struct _OrgMprisMediaPlayer2PlayerIface OrgMprisMediaPlayer2PlayerIface;

struct _OrgMprisMediaPlayer2PlayerIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_next) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_open_uri) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_unnamed_arg0);

  gboolean (*handle_pause) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_play) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_play_pause) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_previous) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_seek) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation,
    gint64 arg_Offset);

  gboolean (*handle_set_position) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_TrackId,
    gint64 arg_Position);

  gboolean (*handle_stop) (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

  gboolean  (*get_can_control) (OrgMprisMediaPlayer2Player *object);

  gboolean  (*get_can_go_next) (OrgMprisMediaPlayer2Player *object);

  gboolean  (*get_can_go_previous) (OrgMprisMediaPlayer2Player *object);

  gboolean  (*get_can_pause) (OrgMprisMediaPlayer2Player *object);

  gboolean  (*get_can_play) (OrgMprisMediaPlayer2Player *object);

  gboolean  (*get_can_seek) (OrgMprisMediaPlayer2Player *object);

  const gchar * (*get_loop_status) (OrgMprisMediaPlayer2Player *object);

  gdouble  (*get_maximum_rate) (OrgMprisMediaPlayer2Player *object);

  GVariant * (*get_metadata) (OrgMprisMediaPlayer2Player *object);

  gdouble  (*get_minimum_rate) (OrgMprisMediaPlayer2Player *object);

  const gchar * (*get_playback_status) (OrgMprisMediaPlayer2Player *object);

  gint64  (*get_position) (OrgMprisMediaPlayer2Player *object);

  gdouble  (*get_rate) (OrgMprisMediaPlayer2Player *object);

  gboolean  (*get_shuffle) (OrgMprisMediaPlayer2Player *object);

  gdouble  (*get_volume) (OrgMprisMediaPlayer2Player *object);

  void (*seeked) (
    OrgMprisMediaPlayer2Player *object,
    gint64 arg_Position);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2Player, g_object_unref)
#endif

GType org_mpris_media_player2_player_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_mpris_media_player2_player_interface_info (void);
guint org_mpris_media_player2_player_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_mpris_media_player2_player_complete_next (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_previous (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_pause (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_play_pause (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_stop (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_play (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_seek (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_set_position (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_player_complete_open_uri (
    OrgMprisMediaPlayer2Player *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void org_mpris_media_player2_player_emit_seeked (
    OrgMprisMediaPlayer2Player *object,
    gint64 arg_Position);



/* D-Bus method calls: */
void org_mpris_media_player2_player_call_next (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_next_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_next_sync (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_previous (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_previous_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_previous_sync (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_pause (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_pause_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_pause_sync (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_play_pause (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_play_pause_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_play_pause_sync (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_stop (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_stop_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_stop_sync (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_play (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_play_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_play_sync (
    OrgMprisMediaPlayer2Player *proxy,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_seek (
    OrgMprisMediaPlayer2Player *proxy,
    gint64 arg_Offset,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_seek_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_seek_sync (
    OrgMprisMediaPlayer2Player *proxy,
    gint64 arg_Offset,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_set_position (
    OrgMprisMediaPlayer2Player *proxy,
    const gchar *arg_TrackId,
    gint64 arg_Position,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_set_position_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_set_position_sync (
    OrgMprisMediaPlayer2Player *proxy,
    const gchar *arg_TrackId,
    gint64 arg_Position,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_player_call_open_uri (
    OrgMprisMediaPlayer2Player *proxy,
    const gchar *arg_unnamed_arg0,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_player_call_open_uri_finish (
    OrgMprisMediaPlayer2Player *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_player_call_open_uri_sync (
    OrgMprisMediaPlayer2Player *proxy,
    const gchar *arg_unnamed_arg0,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *org_mpris_media_player2_player_get_playback_status (OrgMprisMediaPlayer2Player *object);
gchar *org_mpris_media_player2_player_dup_playback_status (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_playback_status (OrgMprisMediaPlayer2Player *object, const gchar *value);

const gchar *org_mpris_media_player2_player_get_loop_status (OrgMprisMediaPlayer2Player *object);
gchar *org_mpris_media_player2_player_dup_loop_status (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_loop_status (OrgMprisMediaPlayer2Player *object, const gchar *value);

gdouble org_mpris_media_player2_player_get_rate (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_rate (OrgMprisMediaPlayer2Player *object, gdouble value);

gboolean org_mpris_media_player2_player_get_shuffle (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_shuffle (OrgMprisMediaPlayer2Player *object, gboolean value);

GVariant *org_mpris_media_player2_player_get_metadata (OrgMprisMediaPlayer2Player *object);
GVariant *org_mpris_media_player2_player_dup_metadata (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_metadata (OrgMprisMediaPlayer2Player *object, GVariant *value);

gdouble org_mpris_media_player2_player_get_volume (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_volume (OrgMprisMediaPlayer2Player *object, gdouble value);

gint64 org_mpris_media_player2_player_get_position (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_position (OrgMprisMediaPlayer2Player *object, gint64 value);

gdouble org_mpris_media_player2_player_get_minimum_rate (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_minimum_rate (OrgMprisMediaPlayer2Player *object, gdouble value);

gdouble org_mpris_media_player2_player_get_maximum_rate (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_maximum_rate (OrgMprisMediaPlayer2Player *object, gdouble value);

gboolean org_mpris_media_player2_player_get_can_go_next (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_can_go_next (OrgMprisMediaPlayer2Player *object, gboolean value);

gboolean org_mpris_media_player2_player_get_can_go_previous (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_can_go_previous (OrgMprisMediaPlayer2Player *object, gboolean value);

gboolean org_mpris_media_player2_player_get_can_play (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_can_play (OrgMprisMediaPlayer2Player *object, gboolean value);

gboolean org_mpris_media_player2_player_get_can_pause (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_can_pause (OrgMprisMediaPlayer2Player *object, gboolean value);

gboolean org_mpris_media_player2_player_get_can_seek (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_can_seek (OrgMprisMediaPlayer2Player *object, gboolean value);

gboolean org_mpris_media_player2_player_get_can_control (OrgMprisMediaPlayer2Player *object);
void org_mpris_media_player2_player_set_can_control (OrgMprisMediaPlayer2Player *object, gboolean value);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY (org_mpris_media_player2_player_proxy_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY, OrgMprisMediaPlayer2PlayerProxy))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY, OrgMprisMediaPlayer2PlayerProxyClass))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY, OrgMprisMediaPlayer2PlayerProxyClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_PROXY))

typedef struct _OrgMprisMediaPlayer2PlayerProxy OrgMprisMediaPlayer2PlayerProxy;
typedef struct _OrgMprisMediaPlayer2PlayerProxyClass OrgMprisMediaPlayer2PlayerProxyClass;
typedef struct _OrgMprisMediaPlayer2PlayerProxyPrivate OrgMprisMediaPlayer2PlayerProxyPrivate;

struct _OrgMprisMediaPlayer2PlayerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgMprisMediaPlayer2PlayerProxyPrivate *priv;
};

struct _OrgMprisMediaPlayer2PlayerProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_mpris_media_player2_player_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2PlayerProxy, g_object_unref)
#endif

void org_mpris_media_player2_player_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2Player *org_mpris_media_player2_player_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2Player *org_mpris_media_player2_player_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_mpris_media_player2_player_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2Player *org_mpris_media_player2_player_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2Player *org_mpris_media_player2_player_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON (org_mpris_media_player2_player_skeleton_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON, OrgMprisMediaPlayer2PlayerSkeleton))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON, OrgMprisMediaPlayer2PlayerSkeletonClass))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON, OrgMprisMediaPlayer2PlayerSkeletonClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYER_SKELETON))

typedef struct _OrgMprisMediaPlayer2PlayerSkeleton OrgMprisMediaPlayer2PlayerSkeleton;
typedef struct _OrgMprisMediaPlayer2PlayerSkeletonClass OrgMprisMediaPlayer2PlayerSkeletonClass;
typedef struct _OrgMprisMediaPlayer2PlayerSkeletonPrivate OrgMprisMediaPlayer2PlayerSkeletonPrivate;

struct _OrgMprisMediaPlayer2PlayerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgMprisMediaPlayer2PlayerSkeletonPrivate *priv;
};

struct _OrgMprisMediaPlayer2PlayerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_mpris_media_player2_player_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2PlayerSkeleton, g_object_unref)
#endif

OrgMprisMediaPlayer2Player *org_mpris_media_player2_player_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.mpris.MediaPlayer2.TrackList */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST (org_mpris_media_player2_track_list_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST, OrgMprisMediaPlayer2TrackList))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST))
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST, OrgMprisMediaPlayer2TrackListIface))

struct _OrgMprisMediaPlayer2TrackList;
typedef struct _OrgMprisMediaPlayer2TrackList OrgMprisMediaPlayer2TrackList;
typedef struct _OrgMprisMediaPlayer2TrackListIface OrgMprisMediaPlayer2TrackListIface;

struct _OrgMprisMediaPlayer2TrackListIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_add_track) (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_Uri,
    const gchar *arg_AfterTrack,
    gboolean arg_SetAsCurrent);

  gboolean (*handle_get_tracks_metadata) (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *arg_TrackIds);

  gboolean (*handle_go_to) (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_TrackId);

  gboolean (*handle_remove_track) (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_TrackId);

  gboolean  (*get_can_edit_tracks) (OrgMprisMediaPlayer2TrackList *object);

  const gchar *const * (*get_tracks) (OrgMprisMediaPlayer2TrackList *object);

  void (*track_added) (
    OrgMprisMediaPlayer2TrackList *object,
    GVariant *arg_Metadata,
    const gchar *arg_AfterTrack);

  void (*track_list_replaced) (
    OrgMprisMediaPlayer2TrackList *object,
    const gchar *const *arg_Tracks,
    const gchar *arg_CurrentTrack);

  void (*track_metadata_changed) (
    OrgMprisMediaPlayer2TrackList *object,
    const gchar *arg_TrackId,
    GVariant *arg_Metadata);

  void (*track_removed) (
    OrgMprisMediaPlayer2TrackList *object,
    const gchar *arg_TrackId);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2TrackList, g_object_unref)
#endif

GType org_mpris_media_player2_track_list_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_mpris_media_player2_track_list_interface_info (void);
guint org_mpris_media_player2_track_list_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_mpris_media_player2_track_list_complete_get_tracks_metadata (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation,
    GVariant *Metadata);

void org_mpris_media_player2_track_list_complete_add_track (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_track_list_complete_remove_track (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_track_list_complete_go_to (
    OrgMprisMediaPlayer2TrackList *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void org_mpris_media_player2_track_list_emit_track_list_replaced (
    OrgMprisMediaPlayer2TrackList *object,
    const gchar *const *arg_Tracks,
    const gchar *arg_CurrentTrack);

void org_mpris_media_player2_track_list_emit_track_added (
    OrgMprisMediaPlayer2TrackList *object,
    GVariant *arg_Metadata,
    const gchar *arg_AfterTrack);

void org_mpris_media_player2_track_list_emit_track_removed (
    OrgMprisMediaPlayer2TrackList *object,
    const gchar *arg_TrackId);

void org_mpris_media_player2_track_list_emit_track_metadata_changed (
    OrgMprisMediaPlayer2TrackList *object,
    const gchar *arg_TrackId,
    GVariant *arg_Metadata);



/* D-Bus method calls: */
void org_mpris_media_player2_track_list_call_get_tracks_metadata (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *const *arg_TrackIds,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_track_list_call_get_tracks_metadata_finish (
    OrgMprisMediaPlayer2TrackList *proxy,
    GVariant **out_Metadata,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_track_list_call_get_tracks_metadata_sync (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *const *arg_TrackIds,
    GVariant **out_Metadata,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_track_list_call_add_track (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *arg_Uri,
    const gchar *arg_AfterTrack,
    gboolean arg_SetAsCurrent,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_track_list_call_add_track_finish (
    OrgMprisMediaPlayer2TrackList *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_track_list_call_add_track_sync (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *arg_Uri,
    const gchar *arg_AfterTrack,
    gboolean arg_SetAsCurrent,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_track_list_call_remove_track (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *arg_TrackId,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_track_list_call_remove_track_finish (
    OrgMprisMediaPlayer2TrackList *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_track_list_call_remove_track_sync (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *arg_TrackId,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_track_list_call_go_to (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *arg_TrackId,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_track_list_call_go_to_finish (
    OrgMprisMediaPlayer2TrackList *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_track_list_call_go_to_sync (
    OrgMprisMediaPlayer2TrackList *proxy,
    const gchar *arg_TrackId,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *const *org_mpris_media_player2_track_list_get_tracks (OrgMprisMediaPlayer2TrackList *object);
gchar **org_mpris_media_player2_track_list_dup_tracks (OrgMprisMediaPlayer2TrackList *object);
void org_mpris_media_player2_track_list_set_tracks (OrgMprisMediaPlayer2TrackList *object, const gchar *const *value);

gboolean org_mpris_media_player2_track_list_get_can_edit_tracks (OrgMprisMediaPlayer2TrackList *object);
void org_mpris_media_player2_track_list_set_can_edit_tracks (OrgMprisMediaPlayer2TrackList *object, gboolean value);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY (org_mpris_media_player2_track_list_proxy_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY, OrgMprisMediaPlayer2TrackListProxy))
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY, OrgMprisMediaPlayer2TrackListProxyClass))
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY, OrgMprisMediaPlayer2TrackListProxyClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_PROXY))

typedef struct _OrgMprisMediaPlayer2TrackListProxy OrgMprisMediaPlayer2TrackListProxy;
typedef struct _OrgMprisMediaPlayer2TrackListProxyClass OrgMprisMediaPlayer2TrackListProxyClass;
typedef struct _OrgMprisMediaPlayer2TrackListProxyPrivate OrgMprisMediaPlayer2TrackListProxyPrivate;

struct _OrgMprisMediaPlayer2TrackListProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgMprisMediaPlayer2TrackListProxyPrivate *priv;
};

struct _OrgMprisMediaPlayer2TrackListProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_mpris_media_player2_track_list_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2TrackListProxy, g_object_unref)
#endif

void org_mpris_media_player2_track_list_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2TrackList *org_mpris_media_player2_track_list_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2TrackList *org_mpris_media_player2_track_list_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_mpris_media_player2_track_list_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2TrackList *org_mpris_media_player2_track_list_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2TrackList *org_mpris_media_player2_track_list_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON (org_mpris_media_player2_track_list_skeleton_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON, OrgMprisMediaPlayer2TrackListSkeleton))
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON, OrgMprisMediaPlayer2TrackListSkeletonClass))
#define ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON, OrgMprisMediaPlayer2TrackListSkeletonClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_TRACK_LIST_SKELETON))

typedef struct _OrgMprisMediaPlayer2TrackListSkeleton OrgMprisMediaPlayer2TrackListSkeleton;
typedef struct _OrgMprisMediaPlayer2TrackListSkeletonClass OrgMprisMediaPlayer2TrackListSkeletonClass;
typedef struct _OrgMprisMediaPlayer2TrackListSkeletonPrivate OrgMprisMediaPlayer2TrackListSkeletonPrivate;

struct _OrgMprisMediaPlayer2TrackListSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgMprisMediaPlayer2TrackListSkeletonPrivate *priv;
};

struct _OrgMprisMediaPlayer2TrackListSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_mpris_media_player2_track_list_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2TrackListSkeleton, g_object_unref)
#endif

OrgMprisMediaPlayer2TrackList *org_mpris_media_player2_track_list_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.mpris.MediaPlayer2.Playlists */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS (org_mpris_media_player2_playlists_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS, OrgMprisMediaPlayer2Playlists))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS, OrgMprisMediaPlayer2PlaylistsIface))

struct _OrgMprisMediaPlayer2Playlists;
typedef struct _OrgMprisMediaPlayer2Playlists OrgMprisMediaPlayer2Playlists;
typedef struct _OrgMprisMediaPlayer2PlaylistsIface OrgMprisMediaPlayer2PlaylistsIface;

struct _OrgMprisMediaPlayer2PlaylistsIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_activate_playlist) (
    OrgMprisMediaPlayer2Playlists *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_PlaylistId);

  gboolean (*handle_get_playlists) (
    OrgMprisMediaPlayer2Playlists *object,
    GDBusMethodInvocation *invocation,
    guint arg_Index,
    guint arg_MaxCount,
    const gchar *arg_Order,
    gboolean arg_ReverseOrder);

  GVariant * (*get_active_playlist) (OrgMprisMediaPlayer2Playlists *object);

  const gchar *const * (*get_orderings) (OrgMprisMediaPlayer2Playlists *object);

  guint  (*get_playlist_count) (OrgMprisMediaPlayer2Playlists *object);

  void (*playlist_changed) (
    OrgMprisMediaPlayer2Playlists *object,
    GVariant *arg_Playlist);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2Playlists, g_object_unref)
#endif

GType org_mpris_media_player2_playlists_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_mpris_media_player2_playlists_interface_info (void);
guint org_mpris_media_player2_playlists_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_mpris_media_player2_playlists_complete_activate_playlist (
    OrgMprisMediaPlayer2Playlists *object,
    GDBusMethodInvocation *invocation);

void org_mpris_media_player2_playlists_complete_get_playlists (
    OrgMprisMediaPlayer2Playlists *object,
    GDBusMethodInvocation *invocation,
    GVariant *Playlists);



/* D-Bus signal emissions functions: */
void org_mpris_media_player2_playlists_emit_playlist_changed (
    OrgMprisMediaPlayer2Playlists *object,
    GVariant *arg_Playlist);



/* D-Bus method calls: */
void org_mpris_media_player2_playlists_call_activate_playlist (
    OrgMprisMediaPlayer2Playlists *proxy,
    const gchar *arg_PlaylistId,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_playlists_call_activate_playlist_finish (
    OrgMprisMediaPlayer2Playlists *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_playlists_call_activate_playlist_sync (
    OrgMprisMediaPlayer2Playlists *proxy,
    const gchar *arg_PlaylistId,
    GCancellable *cancellable,
    GError **error);

void org_mpris_media_player2_playlists_call_get_playlists (
    OrgMprisMediaPlayer2Playlists *proxy,
    guint arg_Index,
    guint arg_MaxCount,
    const gchar *arg_Order,
    gboolean arg_ReverseOrder,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_mpris_media_player2_playlists_call_get_playlists_finish (
    OrgMprisMediaPlayer2Playlists *proxy,
    GVariant **out_Playlists,
    GAsyncResult *res,
    GError **error);

gboolean org_mpris_media_player2_playlists_call_get_playlists_sync (
    OrgMprisMediaPlayer2Playlists *proxy,
    guint arg_Index,
    guint arg_MaxCount,
    const gchar *arg_Order,
    gboolean arg_ReverseOrder,
    GVariant **out_Playlists,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint org_mpris_media_player2_playlists_get_playlist_count (OrgMprisMediaPlayer2Playlists *object);
void org_mpris_media_player2_playlists_set_playlist_count (OrgMprisMediaPlayer2Playlists *object, guint value);

const gchar *const *org_mpris_media_player2_playlists_get_orderings (OrgMprisMediaPlayer2Playlists *object);
gchar **org_mpris_media_player2_playlists_dup_orderings (OrgMprisMediaPlayer2Playlists *object);
void org_mpris_media_player2_playlists_set_orderings (OrgMprisMediaPlayer2Playlists *object, const gchar *const *value);

GVariant *org_mpris_media_player2_playlists_get_active_playlist (OrgMprisMediaPlayer2Playlists *object);
GVariant *org_mpris_media_player2_playlists_dup_active_playlist (OrgMprisMediaPlayer2Playlists *object);
void org_mpris_media_player2_playlists_set_active_playlist (OrgMprisMediaPlayer2Playlists *object, GVariant *value);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY (org_mpris_media_player2_playlists_proxy_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY, OrgMprisMediaPlayer2PlaylistsProxy))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY, OrgMprisMediaPlayer2PlaylistsProxyClass))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY, OrgMprisMediaPlayer2PlaylistsProxyClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_PROXY))

typedef struct _OrgMprisMediaPlayer2PlaylistsProxy OrgMprisMediaPlayer2PlaylistsProxy;
typedef struct _OrgMprisMediaPlayer2PlaylistsProxyClass OrgMprisMediaPlayer2PlaylistsProxyClass;
typedef struct _OrgMprisMediaPlayer2PlaylistsProxyPrivate OrgMprisMediaPlayer2PlaylistsProxyPrivate;

struct _OrgMprisMediaPlayer2PlaylistsProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgMprisMediaPlayer2PlaylistsProxyPrivate *priv;
};

struct _OrgMprisMediaPlayer2PlaylistsProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_mpris_media_player2_playlists_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2PlaylistsProxy, g_object_unref)
#endif

void org_mpris_media_player2_playlists_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2Playlists *org_mpris_media_player2_playlists_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2Playlists *org_mpris_media_player2_playlists_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_mpris_media_player2_playlists_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgMprisMediaPlayer2Playlists *org_mpris_media_player2_playlists_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgMprisMediaPlayer2Playlists *org_mpris_media_player2_playlists_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON (org_mpris_media_player2_playlists_skeleton_get_type ())
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON, OrgMprisMediaPlayer2PlaylistsSkeleton))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON, OrgMprisMediaPlayer2PlaylistsSkeletonClass))
#define ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON, OrgMprisMediaPlayer2PlaylistsSkeletonClass))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON))
#define IS_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_MPRIS_MEDIA_PLAYER2_PLAYLISTS_SKELETON))

typedef struct _OrgMprisMediaPlayer2PlaylistsSkeleton OrgMprisMediaPlayer2PlaylistsSkeleton;
typedef struct _OrgMprisMediaPlayer2PlaylistsSkeletonClass OrgMprisMediaPlayer2PlaylistsSkeletonClass;
typedef struct _OrgMprisMediaPlayer2PlaylistsSkeletonPrivate OrgMprisMediaPlayer2PlaylistsSkeletonPrivate;

struct _OrgMprisMediaPlayer2PlaylistsSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgMprisMediaPlayer2PlaylistsSkeletonPrivate *priv;
};

struct _OrgMprisMediaPlayer2PlaylistsSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_mpris_media_player2_playlists_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgMprisMediaPlayer2PlaylistsSkeleton, g_object_unref)
#endif

OrgMprisMediaPlayer2Playlists *org_mpris_media_player2_playlists_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.DBus.Properties */

#define TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES (org_freedesktop_dbus_properties_get_type ())
#define ORG_FREEDESKTOP_DBUS_PROPERTIES(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES, OrgFreedesktopDBusProperties))
#define IS_ORG_FREEDESKTOP_DBUS_PROPERTIES(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES))
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES, OrgFreedesktopDBusPropertiesIface))

struct _OrgFreedesktopDBusProperties;
typedef struct _OrgFreedesktopDBusProperties OrgFreedesktopDBusProperties;
typedef struct _OrgFreedesktopDBusPropertiesIface OrgFreedesktopDBusPropertiesIface;

struct _OrgFreedesktopDBusPropertiesIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_get) (
    OrgFreedesktopDBusProperties *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_interface_name,
    const gchar *arg_property_name);

  gboolean (*handle_get_all) (
    OrgFreedesktopDBusProperties *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_interface_name);

  gboolean (*handle_set) (
    OrgFreedesktopDBusProperties *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_interface_name,
    const gchar *arg_property_name,
    GVariant *arg_value);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusProperties, g_object_unref)
#endif

GType org_freedesktop_dbus_properties_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_freedesktop_dbus_properties_interface_info (void);
guint org_freedesktop_dbus_properties_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_freedesktop_dbus_properties_complete_get (
    OrgFreedesktopDBusProperties *object,
    GDBusMethodInvocation *invocation,
    GVariant *value);

void org_freedesktop_dbus_properties_complete_set (
    OrgFreedesktopDBusProperties *object,
    GDBusMethodInvocation *invocation);

void org_freedesktop_dbus_properties_complete_get_all (
    OrgFreedesktopDBusProperties *object,
    GDBusMethodInvocation *invocation,
    GVariant *values);



/* D-Bus method calls: */
void org_freedesktop_dbus_properties_call_get (
    OrgFreedesktopDBusProperties *proxy,
    const gchar *arg_interface_name,
    const gchar *arg_property_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_freedesktop_dbus_properties_call_get_finish (
    OrgFreedesktopDBusProperties *proxy,
    GVariant **out_value,
    GAsyncResult *res,
    GError **error);

gboolean org_freedesktop_dbus_properties_call_get_sync (
    OrgFreedesktopDBusProperties *proxy,
    const gchar *arg_interface_name,
    const gchar *arg_property_name,
    GVariant **out_value,
    GCancellable *cancellable,
    GError **error);

void org_freedesktop_dbus_properties_call_set (
    OrgFreedesktopDBusProperties *proxy,
    const gchar *arg_interface_name,
    const gchar *arg_property_name,
    GVariant *arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_freedesktop_dbus_properties_call_set_finish (
    OrgFreedesktopDBusProperties *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_freedesktop_dbus_properties_call_set_sync (
    OrgFreedesktopDBusProperties *proxy,
    const gchar *arg_interface_name,
    const gchar *arg_property_name,
    GVariant *arg_value,
    GCancellable *cancellable,
    GError **error);

void org_freedesktop_dbus_properties_call_get_all (
    OrgFreedesktopDBusProperties *proxy,
    const gchar *arg_interface_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_freedesktop_dbus_properties_call_get_all_finish (
    OrgFreedesktopDBusProperties *proxy,
    GVariant **out_values,
    GAsyncResult *res,
    GError **error);

gboolean org_freedesktop_dbus_properties_call_get_all_sync (
    OrgFreedesktopDBusProperties *proxy,
    const gchar *arg_interface_name,
    GVariant **out_values,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY (org_freedesktop_dbus_properties_proxy_get_type ())
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY, OrgFreedesktopDBusPropertiesProxy))
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY, OrgFreedesktopDBusPropertiesProxyClass))
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY, OrgFreedesktopDBusPropertiesProxyClass))
#define IS_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY))
#define IS_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_PROXY))

typedef struct _OrgFreedesktopDBusPropertiesProxy OrgFreedesktopDBusPropertiesProxy;
typedef struct _OrgFreedesktopDBusPropertiesProxyClass OrgFreedesktopDBusPropertiesProxyClass;
typedef struct _OrgFreedesktopDBusPropertiesProxyPrivate OrgFreedesktopDBusPropertiesProxyPrivate;

struct _OrgFreedesktopDBusPropertiesProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgFreedesktopDBusPropertiesProxyPrivate *priv;
};

struct _OrgFreedesktopDBusPropertiesProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_freedesktop_dbus_properties_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusPropertiesProxy, g_object_unref)
#endif

void org_freedesktop_dbus_properties_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgFreedesktopDBusProperties *org_freedesktop_dbus_properties_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgFreedesktopDBusProperties *org_freedesktop_dbus_properties_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_freedesktop_dbus_properties_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgFreedesktopDBusProperties *org_freedesktop_dbus_properties_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgFreedesktopDBusProperties *org_freedesktop_dbus_properties_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON (org_freedesktop_dbus_properties_skeleton_get_type ())
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON, OrgFreedesktopDBusPropertiesSkeleton))
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON, OrgFreedesktopDBusPropertiesSkeletonClass))
#define ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON, OrgFreedesktopDBusPropertiesSkeletonClass))
#define IS_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON))
#define IS_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_FREEDESKTOP_DBUS_PROPERTIES_SKELETON))

typedef struct _OrgFreedesktopDBusPropertiesSkeleton OrgFreedesktopDBusPropertiesSkeleton;
typedef struct _OrgFreedesktopDBusPropertiesSkeletonClass OrgFreedesktopDBusPropertiesSkeletonClass;
typedef struct _OrgFreedesktopDBusPropertiesSkeletonPrivate OrgFreedesktopDBusPropertiesSkeletonPrivate;

struct _OrgFreedesktopDBusPropertiesSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgFreedesktopDBusPropertiesSkeletonPrivate *priv;
};

struct _OrgFreedesktopDBusPropertiesSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_freedesktop_dbus_properties_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusPropertiesSkeleton, g_object_unref)
#endif

OrgFreedesktopDBusProperties *org_freedesktop_dbus_properties_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.DBus.Introspectable */

#define TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE (org_freedesktop_dbus_introspectable_get_type ())
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE, OrgFreedesktopDBusIntrospectable))
#define IS_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE))
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE, OrgFreedesktopDBusIntrospectableIface))

struct _OrgFreedesktopDBusIntrospectable;
typedef struct _OrgFreedesktopDBusIntrospectable OrgFreedesktopDBusIntrospectable;
typedef struct _OrgFreedesktopDBusIntrospectableIface OrgFreedesktopDBusIntrospectableIface;

struct _OrgFreedesktopDBusIntrospectableIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_introspect) (
    OrgFreedesktopDBusIntrospectable *object,
    GDBusMethodInvocation *invocation);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusIntrospectable, g_object_unref)
#endif

GType org_freedesktop_dbus_introspectable_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_freedesktop_dbus_introspectable_interface_info (void);
guint org_freedesktop_dbus_introspectable_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_freedesktop_dbus_introspectable_complete_introspect (
    OrgFreedesktopDBusIntrospectable *object,
    GDBusMethodInvocation *invocation,
    const gchar *xml_data);



/* D-Bus method calls: */
void org_freedesktop_dbus_introspectable_call_introspect (
    OrgFreedesktopDBusIntrospectable *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_freedesktop_dbus_introspectable_call_introspect_finish (
    OrgFreedesktopDBusIntrospectable *proxy,
    gchar **out_xml_data,
    GAsyncResult *res,
    GError **error);

gboolean org_freedesktop_dbus_introspectable_call_introspect_sync (
    OrgFreedesktopDBusIntrospectable *proxy,
    gchar **out_xml_data,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY (org_freedesktop_dbus_introspectable_proxy_get_type ())
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY, OrgFreedesktopDBusIntrospectableProxy))
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY, OrgFreedesktopDBusIntrospectableProxyClass))
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY, OrgFreedesktopDBusIntrospectableProxyClass))
#define IS_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY))
#define IS_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_PROXY))

typedef struct _OrgFreedesktopDBusIntrospectableProxy OrgFreedesktopDBusIntrospectableProxy;
typedef struct _OrgFreedesktopDBusIntrospectableProxyClass OrgFreedesktopDBusIntrospectableProxyClass;
typedef struct _OrgFreedesktopDBusIntrospectableProxyPrivate OrgFreedesktopDBusIntrospectableProxyPrivate;

struct _OrgFreedesktopDBusIntrospectableProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgFreedesktopDBusIntrospectableProxyPrivate *priv;
};

struct _OrgFreedesktopDBusIntrospectableProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_freedesktop_dbus_introspectable_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusIntrospectableProxy, g_object_unref)
#endif

void org_freedesktop_dbus_introspectable_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgFreedesktopDBusIntrospectable *org_freedesktop_dbus_introspectable_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgFreedesktopDBusIntrospectable *org_freedesktop_dbus_introspectable_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_freedesktop_dbus_introspectable_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgFreedesktopDBusIntrospectable *org_freedesktop_dbus_introspectable_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgFreedesktopDBusIntrospectable *org_freedesktop_dbus_introspectable_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON (org_freedesktop_dbus_introspectable_skeleton_get_type ())
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON, OrgFreedesktopDBusIntrospectableSkeleton))
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON, OrgFreedesktopDBusIntrospectableSkeletonClass))
#define ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON, OrgFreedesktopDBusIntrospectableSkeletonClass))
#define IS_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON))
#define IS_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_FREEDESKTOP_DBUS_INTROSPECTABLE_SKELETON))

typedef struct _OrgFreedesktopDBusIntrospectableSkeleton OrgFreedesktopDBusIntrospectableSkeleton;
typedef struct _OrgFreedesktopDBusIntrospectableSkeletonClass OrgFreedesktopDBusIntrospectableSkeletonClass;
typedef struct _OrgFreedesktopDBusIntrospectableSkeletonPrivate OrgFreedesktopDBusIntrospectableSkeletonPrivate;

struct _OrgFreedesktopDBusIntrospectableSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgFreedesktopDBusIntrospectableSkeletonPrivate *priv;
};

struct _OrgFreedesktopDBusIntrospectableSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_freedesktop_dbus_introspectable_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusIntrospectableSkeleton, g_object_unref)
#endif

OrgFreedesktopDBusIntrospectable *org_freedesktop_dbus_introspectable_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.DBus.Peer */

#define TYPE_ORG_FREEDESKTOP_DBUS_PEER (org_freedesktop_dbus_peer_get_type ())
#define ORG_FREEDESKTOP_DBUS_PEER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER, OrgFreedesktopDBusPeer))
#define IS_ORG_FREEDESKTOP_DBUS_PEER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER))
#define ORG_FREEDESKTOP_DBUS_PEER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER, OrgFreedesktopDBusPeerIface))

struct _OrgFreedesktopDBusPeer;
typedef struct _OrgFreedesktopDBusPeer OrgFreedesktopDBusPeer;
typedef struct _OrgFreedesktopDBusPeerIface OrgFreedesktopDBusPeerIface;

struct _OrgFreedesktopDBusPeerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_get_machine_id) (
    OrgFreedesktopDBusPeer *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_ping) (
    OrgFreedesktopDBusPeer *object,
    GDBusMethodInvocation *invocation);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusPeer, g_object_unref)
#endif

GType org_freedesktop_dbus_peer_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *org_freedesktop_dbus_peer_interface_info (void);
guint org_freedesktop_dbus_peer_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void org_freedesktop_dbus_peer_complete_ping (
    OrgFreedesktopDBusPeer *object,
    GDBusMethodInvocation *invocation);

void org_freedesktop_dbus_peer_complete_get_machine_id (
    OrgFreedesktopDBusPeer *object,
    GDBusMethodInvocation *invocation,
    const gchar *machine_uuid);



/* D-Bus method calls: */
void org_freedesktop_dbus_peer_call_ping (
    OrgFreedesktopDBusPeer *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_freedesktop_dbus_peer_call_ping_finish (
    OrgFreedesktopDBusPeer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean org_freedesktop_dbus_peer_call_ping_sync (
    OrgFreedesktopDBusPeer *proxy,
    GCancellable *cancellable,
    GError **error);

void org_freedesktop_dbus_peer_call_get_machine_id (
    OrgFreedesktopDBusPeer *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean org_freedesktop_dbus_peer_call_get_machine_id_finish (
    OrgFreedesktopDBusPeer *proxy,
    gchar **out_machine_uuid,
    GAsyncResult *res,
    GError **error);

gboolean org_freedesktop_dbus_peer_call_get_machine_id_sync (
    OrgFreedesktopDBusPeer *proxy,
    gchar **out_machine_uuid,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define TYPE_ORG_FREEDESKTOP_DBUS_PEER_PROXY (org_freedesktop_dbus_peer_proxy_get_type ())
#define ORG_FREEDESKTOP_DBUS_PEER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER_PROXY, OrgFreedesktopDBusPeerProxy))
#define ORG_FREEDESKTOP_DBUS_PEER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_FREEDESKTOP_DBUS_PEER_PROXY, OrgFreedesktopDBusPeerProxyClass))
#define ORG_FREEDESKTOP_DBUS_PEER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER_PROXY, OrgFreedesktopDBusPeerProxyClass))
#define IS_ORG_FREEDESKTOP_DBUS_PEER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER_PROXY))
#define IS_ORG_FREEDESKTOP_DBUS_PEER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_FREEDESKTOP_DBUS_PEER_PROXY))

typedef struct _OrgFreedesktopDBusPeerProxy OrgFreedesktopDBusPeerProxy;
typedef struct _OrgFreedesktopDBusPeerProxyClass OrgFreedesktopDBusPeerProxyClass;
typedef struct _OrgFreedesktopDBusPeerProxyPrivate OrgFreedesktopDBusPeerProxyPrivate;

struct _OrgFreedesktopDBusPeerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  OrgFreedesktopDBusPeerProxyPrivate *priv;
};

struct _OrgFreedesktopDBusPeerProxyClass
{
  GDBusProxyClass parent_class;
};

GType org_freedesktop_dbus_peer_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusPeerProxy, g_object_unref)
#endif

void org_freedesktop_dbus_peer_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgFreedesktopDBusPeer *org_freedesktop_dbus_peer_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
OrgFreedesktopDBusPeer *org_freedesktop_dbus_peer_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void org_freedesktop_dbus_peer_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
OrgFreedesktopDBusPeer *org_freedesktop_dbus_peer_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
OrgFreedesktopDBusPeer *org_freedesktop_dbus_peer_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TYPE_ORG_FREEDESKTOP_DBUS_PEER_SKELETON (org_freedesktop_dbus_peer_skeleton_get_type ())
#define ORG_FREEDESKTOP_DBUS_PEER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER_SKELETON, OrgFreedesktopDBusPeerSkeleton))
#define ORG_FREEDESKTOP_DBUS_PEER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TYPE_ORG_FREEDESKTOP_DBUS_PEER_SKELETON, OrgFreedesktopDBusPeerSkeletonClass))
#define ORG_FREEDESKTOP_DBUS_PEER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER_SKELETON, OrgFreedesktopDBusPeerSkeletonClass))
#define IS_ORG_FREEDESKTOP_DBUS_PEER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TYPE_ORG_FREEDESKTOP_DBUS_PEER_SKELETON))
#define IS_ORG_FREEDESKTOP_DBUS_PEER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TYPE_ORG_FREEDESKTOP_DBUS_PEER_SKELETON))

typedef struct _OrgFreedesktopDBusPeerSkeleton OrgFreedesktopDBusPeerSkeleton;
typedef struct _OrgFreedesktopDBusPeerSkeletonClass OrgFreedesktopDBusPeerSkeletonClass;
typedef struct _OrgFreedesktopDBusPeerSkeletonPrivate OrgFreedesktopDBusPeerSkeletonPrivate;

struct _OrgFreedesktopDBusPeerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  OrgFreedesktopDBusPeerSkeletonPrivate *priv;
};

struct _OrgFreedesktopDBusPeerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType org_freedesktop_dbus_peer_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (OrgFreedesktopDBusPeerSkeleton, g_object_unref)
#endif

OrgFreedesktopDBusPeer *org_freedesktop_dbus_peer_skeleton_new (void);


G_END_DECLS

#endif /* __PLAYERCTL_GENERATED_H__ */
