
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "playerctl-enum-types.h"
#include "playerctl.h"
#include "playerctl-player.h"
#include "playerctl-player-manager.h"
#include "playerctl-player-name.h"
#include "playerctl-version.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "playerctl-player-name.h" */

GType
playerctl_source_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PLAYERCTL_SOURCE_NONE), "PLAYERCTL_SOURCE_NONE", "none" },
        { C_ENUM (PLAYERCTL_SOURCE_DBUS_SESSION), "PLAYERCTL_SOURCE_DBUS_SESSION", "dbus-session" },
        { C_ENUM (PLAYERCTL_SOURCE_DBUS_SYSTEM), "PLAYERCTL_SOURCE_DBUS_SYSTEM", "dbus-system" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PlayerctlSource"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "playerctl-player.h" */

GType
playerctl_playback_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PLAYERCTL_PLAYBACK_STATUS_PLAYING), "PLAYERCTL_PLAYBACK_STATUS_PLAYING", "Playing" },
        { C_ENUM (PLAYERCTL_PLAYBACK_STATUS_PAUSED), "PLAYERCTL_PLAYBACK_STATUS_PAUSED", "Paused" },
        { C_ENUM (PLAYERCTL_PLAYBACK_STATUS_STOPPED), "PLAYERCTL_PLAYBACK_STATUS_STOPPED", "Stopped" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PlayerctlPlaybackStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
playerctl_loop_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (PLAYERCTL_LOOP_STATUS_NONE), "PLAYERCTL_LOOP_STATUS_NONE", "None" },
        { C_ENUM (PLAYERCTL_LOOP_STATUS_TRACK), "PLAYERCTL_LOOP_STATUS_TRACK", "Track" },
        { C_ENUM (PLAYERCTL_LOOP_STATUS_PLAYLIST), "PLAYERCTL_LOOP_STATUS_PLAYLIST", "playlist" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("PlayerctlLoopStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

