
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "playerctl-player-name.h" */

GType playerctl_source_get_type (void);
#define PLAYERCTL_TYPE_SOURCE (playerctl_source_get_type())

/* enumerations from "playerctl-player.h" */

GType playerctl_playback_status_get_type (void);
#define PLAYERCTL_TYPE_PLAYBACK_STATUS (playerctl_playback_status_get_type())

GType playerctl_loop_status_get_type (void);
#define PLAYERCTL_TYPE_LOOP_STATUS (playerctl_loop_status_get_type())

G_END_DECLS

/* Generated data ends here */

